@@ -1,7 +1,7 @@
 /*
  * RS-DBI.c
  *
- * $Id: RS-DBI.c 212 2011-11-22 10:58:50Z tomoakin@kenroku.kanazawa-u.ac.jp $
+ * $Id: RS-DBI.c 237 2012-10-04 14:54:29Z tomoakin@kenroku.kanazawa-u.ac.jp $
  *
  * This package was developed as a part of Summer of Code program organized by Google.
  * Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
@@ -679,15 +679,7 @@ RS_DBI_copyfields(RS_DBI_fields * flds)
     for (j = 0; j < n; j++) {
         lengths[j] = (Sint) num_fields;
     }
-    S_fields = RS_DBI_createNamedList(desc, types, lengths, n);
-#ifndef USING_R
-    if (IS_LIST(S_fields)) {
-        S_fields = AS_LIST(S_fields);
-    }
-    else {
-        RS_DBI_errorMessage("internal error in RS_DBI_copyfields: could not alloc named list", RS_DBI_ERROR);
-    }
-#endif
+    PROTECT(S_fields = RS_DBI_createNamedList(desc, types, lengths, n));
     /* copy contentes from flds into an R/S list */
     for (i = 0; i < num_fields; i++) {
         SET_LST_CHR_EL(S_fields, 0, i, C_S_CPY(flds->name[i]));
@@ -699,7 +691,7 @@ RS_DBI_copyfields(RS_DBI_fields * flds)
         LST_INT_EL(S_fields, 6, i) = (Sint) flds->isVarLength[i];
         LST_INT_EL(S_fields, 7, i) = (Sint) flds->nullOk[i];
     }
-
+    UNPROTECT(1);
     return S_fields;
 }
 
@@ -753,14 +745,12 @@ RS_DBI_SclassNames(s_object * type)
     Sint *typeCodes;
     Sint n;
     int i;
-    char *s;
+    const char *s;
 
-    if (type == S_NULL_ENTRY) {
-        RS_DBI_errorMessage("internal error in RS_DBI_SclassNames: input S types must be nonNULL", RS_DBI_ERROR);
-    }
+    PROTECT(type = AS_INTEGER(type));
     n = LENGTH(type);
     typeCodes = INTEGER_DATA(type);
-    MEM_PROTECT(typeNames = NEW_CHARACTER(n));
+    PROTECT(typeNames = NEW_CHARACTER(n));
     for (i = 0; i < n; i++) {
         s = RS_DBI_getTypeName(typeCodes[i], RS_dataTypeTable);
         if (!s) {
@@ -768,7 +758,7 @@ RS_DBI_SclassNames(s_object * type)
         }
         SET_CHR_EL(typeNames, i, C_S_CPY(s));
     }
-    MEM_UNPROTECT(1);
+    UNPROTECT(2);
     return typeNames;
 }
 
@@ -839,7 +829,7 @@ RS_DBI_getConnection(Con_Handle * conHandle)
         RS_DBI_errorMessage("internal error in RS_DBI_getConnection: corrupt connection handle", RS_DBI_ERROR);
     }
     if (!mgr->connections[indx]) {
-        RS_DBI_errorMessage("internal error in RS_DBI_getConnection: corrupt connection  object", RS_DBI_ERROR);
+        RS_DBI_errorMessage("internal error in RS_DBI_getConnection: corrupt connection object", RS_DBI_ERROR);
     }
     return mgr->connections[indx];
 }
@@ -1015,15 +1005,7 @@ RS_DBI_managerInfo(Mgr_Handle * mgrHandle)
     num_con = (Sint) mgr->num_con;
     mgrLen[0] = num_con;
 
-    output = RS_DBI_createNamedList(mgrDesc, mgrType, mgrLen, n);
-#ifndef USING_R
-    if (IS_LIST(output)) {
-        output = AS_LIST(output);
-    }
-    else {
-        RS_DBI_errorMessage("internal error: could not alloc named list", RS_DBI_ERROR);
-    }
-#endif
+    PROTECT(output = RS_DBI_createNamedList(mgrDesc, mgrType, mgrLen, n));
     for (i = 0; i < num_con; i++) {
         LST_INT_EL(output, 0, i) = (Sint) mgr->connectionIds[i];
     }
@@ -1033,7 +1015,7 @@ RS_DBI_managerInfo(Mgr_Handle * mgrHandle)
     LST_INT_EL(output, 4, 0) = (Sint) mgr->num_con;
     LST_INT_EL(output, 5, 0) = (Sint) mgr->counter;
     SET_LST_CHR_EL(output, 6, 0, C_S_CPY("NA"));        /* client versions? */
-
+    UNPROTECT(1);
     return output;
 }
 
@@ -1061,15 +1043,7 @@ RS_DBI_connectionInfo(Con_Handle * conHandle)
     con = RS_DBI_getConnection(conHandle);
     conLen[6] = con->num_res;   /* number of resultSets opened *//* NOTE: changed from 7 to 6 */
 
-    output = RS_DBI_createNamedList(conDesc, conType, conLen, n);
-#ifndef USING_R
-    if (IS_LIST(output)) {
-        output = AS_LIST(output);
-    }
-    else {
-        RS_DBI_errorMessage("internal error in RS_DBI_connectionInfo: could not alloc named list", RS_DBI_ERROR);
-    }
-#endif
+    PROTECT(output = RS_DBI_createNamedList(conDesc, conType, conLen, n));
     /* dummy */
     SET_LST_CHR_EL(output, 0, 0, C_S_CPY("NA"));        /* host */
     SET_LST_CHR_EL(output, 1, 0, C_S_CPY("NA"));        /* dbname */
@@ -1083,6 +1057,7 @@ RS_DBI_connectionInfo(Con_Handle * conHandle)
     for (i = 0; i < con->num_res; i++) {
         LST_INT_EL(output, 6, (Sint) i) = con->resultSetIds[i];
     }
+    UNPROTECT(1);
     return output;
 }
 
@@ -1102,28 +1077,20 @@ RS_DBI_resultSetInfo(Res_Handle * rsHandle)
 
     result = RS_DBI_getResultSet(rsHandle);
     if (result->fields) {
-        flds = RS_DBI_copyfields(result->fields);
+        PROTECT(flds = RS_DBI_copyfields(result->fields));
     }
     else {
-        flds = S_NULL_ENTRY;
+        PROTECT(flds = S_NULL_ENTRY);
     }
 
-    output = RS_DBI_createNamedList(rsDesc, rsType, rsLen, n);
-#ifndef USING_R
-    if (IS_LIST(output)) {
-        output = AS_LIST(output);
-    }
-    else {
-        RS_DBI_errorMessage("internal error in RS_DBI_resultSetInfo: could not alloc named list", RS_DBI_ERROR);
-    }
-#endif
+    PROTECT(output = RS_DBI_createNamedList(rsDesc, rsType, rsLen, n));
     SET_LST_CHR_EL(output, 0, 0, C_S_CPY(result->statement));
     LST_INT_EL(output, 1, 0) = result->isSelect;
     LST_INT_EL(output, 2, 0) = result->rowsAffected;
     LST_INT_EL(output, 3, 0) = result->rowCount;
     LST_INT_EL(output, 4, 0) = result->completed;
     SET_ELEMENT(LST_EL(output, 5), (Sint) 0, flds);
-
+    UNPROTECT(2);
     return output;
 }
 
@@ -1147,15 +1114,7 @@ RS_DBI_getFieldDescriptions(RS_DBI_fields * flds)
     for (j = 0; j < n; j++) {
         lengths[j] = (Sint) num_fields;
     }
-    S_fields = RS_DBI_createNamedList(desc, types, lengths, n);
-#ifndef USING_R
-    if (IS_LIST(S_fields)) {
-        S_fields = AS_LIST(S_fields);
-    }
-    else {
-        RS_DBI_errorMessage("internal error in RS_DBI_getFieldDescription: could not alloc named list", RS_DBI_ERROR);
-    }
-#endif
+    PROTECT(S_fields = RS_DBI_createNamedList(desc, types, lengths, n));
     /* copy contentes from flds into an R/S list */
     for (i = 0; i < (Sint) num_fields; i++) {
         SET_LST_CHR_EL(S_fields, 0, i, C_S_CPY(flds->name[i]));
@@ -1166,13 +1125,13 @@ RS_DBI_getFieldDescriptions(RS_DBI_fields * flds)
         LST_INT_EL(S_fields, 5, i) = (Sint) flds->scale[i];
         LST_INT_EL(S_fields, 6, i) = (Sint) flds->nullOk[i];
     }
-
+    UNPROTECT(1);
     return (S_fields);
 }
 
 /* given a type id return its human-readable name.
  * We define an RS_DBI_dataTypeTable */
-char *
+const char *
 RS_DBI_getTypeName(Sint t, const struct data_types table[])
 {
     int i;
@@ -1185,7 +1144,7 @@ RS_DBI_getTypeName(Sint t, const struct data_types table[])
     }
     sprintf(buf, "unknown (%ld)", (long) t);
     RS_DBI_errorMessage(buf, RS_DBI_WARNING);
-    return (char *) 0;          /* for -Wall */
+    return "UNKNOWN";
 }
 
 /* Translate R/S identifiers (and only R/S names!!!) into
@@ -1269,16 +1228,6 @@ RS_na_set(void *ptr, Stype type)
         d = (double *) ptr;
         *d = NA_REAL;
         break;
-#if 0
-/* this is apparently wrong code and currently NA_SET with STRING_TYPE is not used in the code*/
-    case STRING_TYPE:
-        {
-          char *c;
-          c = (char *) ptr;
-          c = (char *) CHR_EL(NA_STRING, 0);      /*NOTE: modified the RHS by adding cating to (char*) */
-          break;
-        }
-#endif
     }
 }
 int
